@@ -3,7 +3,7 @@ import sys
 import matplotlib.pyplot as plt
 import numpy as np
 import random
-
+import time
 try:
     QT = 4
     import PyQt4.QtGui as QtG
@@ -503,15 +503,20 @@ class MainWindow(QtG.QMainWindow):
         pure_filename = os.path.splitext(os.path.basename(self.current_image_path))[0]
         if pure_filename in self.box_dictionary:
             self.rectangles = self.box_dictionary[pure_filename]
-            self.delete_all_patches(self.rectangles)
+            self.delete_all_patches(self.rectangles, update=True)
             self.draw_all_patches(self.rectangles)
             self._draw_all_boxes()
 
-    def delete_all_patches(self, rects):
+    def delete_all_patches(self, rects, update=False):
+        state = self.get_filter_state()
         for box in rects:
-            box.set_visible(False)
-            if box.is_figure_set():
-                box.remove()
+            if self.check_if_should_be_visible(box)==False or update==False:
+                rect = box.getRect()
+                rect.set_visible(False)
+                if rect.is_figure_set():
+                    rect.remove()
+            if not self.filter_tuple_is_equal(self.get_filter_state(),state):
+                break
 
     def write_box_files(self):
         """
@@ -552,10 +557,11 @@ class MainWindow(QtG.QMainWindow):
                     box for box in rectangles if self.check_if_should_be_visible(box)
                 ]
                 #if self.use_estimated_size_checkbox.isChecked():
-            for rect in rectangles:
+            real_rects = [rect.getRect() for rect in rectangles]
+            for rect in real_rects:
                 self.resize_box(rect, self.boxsize)
-            num_writtin_part = num_writtin_part + len(rectangles)
-            CoordsIO.write_boxfile_manager(box_file_path, rectangles)
+            num_writtin_part = num_writtin_part + len(real_rects)
+            CoordsIO.write_boxfile_manager(box_file_path, real_rects)
             self.unsaved_changes = False
         print(num_writtin_part, "particles written.")
 
@@ -708,7 +714,6 @@ class MainWindow(QtG.QMainWindow):
                 else:
                     self.box_dictionary[dict_entry_name] = rects
 
-        print("Total time", t.time() - t_start)
         if self.is_cbox:
             self.conf_thresh_line.setEnabled(True)
             self.conf_thresh_slide.setEnabled(True)
@@ -748,7 +753,18 @@ class MainWindow(QtG.QMainWindow):
         if filaments_imported > 0:
             print("Total imported filaments: ", filaments_imported)
 
+    def get_filter_state(self):
+        lowersize = int(float(self.lower_size_thresh_line.text()))
+        uppersize = int(float(self.lower_size_thresh_line.text()))
+        conf = float(self.conf_thresh_slide.value())
+        return (lowersize,uppersize,conf)
+
+    def filter_tuple_is_equal(self,a,b):
+        return a[0]==b[0] and a[1]==b[1] and a[2] == b[2]
+
     def update_tree_boxsizes(self, update_current=False):
+        state = self.get_filter_state()
+
         def update(boxes, item):
 
             res = [self.check_if_should_be_visible(box) for box in boxes]
@@ -762,19 +778,20 @@ class MainWindow(QtG.QMainWindow):
             update(self.box_dictionary[filename], item)
 
         else:
+            start = time.time()
             root = self.tree.invisibleRootItem().child(0)
             child_count = root.childCount()
 
             for i in range(child_count):
                 QtCore.QCoreApplication.instance().processEvents()
+                if not self.filter_tuple_is_equal(self.get_filter_state(),state):
+                    break
                 item = root.child(i)
                 filename = os.path.splitext(item.text(0))[0]
                 if filename in self.box_dictionary:
                     # num_box = sum(map(lambda x: self.check_if_should_be_visible(x), self.box_dictionary[filename]))
                     # item.setText(1, str(num_box))
                     update(self.box_dictionary[filename], item)
-
-
     def get_all_loaded_filesnames(self):
         root = self.tree.invisibleRootItem().child(0)
         child_count = root.childCount()
@@ -872,13 +889,18 @@ class MainWindow(QtG.QMainWindow):
             and box.est_size >= self.lower_size_thresh
         )
 
-    def draw_all_patches(self, rect):
-        for box in rect:
-            should_be_visible = self.check_if_should_be_visible(box)
-            if should_be_visible:
-                box.set_visible(True)
-                if box not in self.ax.patches:
-                    self.ax.add_patch(box)
+
+    def draw_all_patches(self, rects):
+        state = self.get_filter_state()
+        visible_rects = [box.getRect() for box in rects if self.check_if_should_be_visible(box)]
+        for rect in visible_rects:
+            if rect.get_visible()==False:
+                rect.set_visible(True)
+            if rect not in self.ax.patches:
+                self.ax.add_patch(rect)
+            if not self.filter_tuple_is_equal(self.get_filter_state(),state):
+                break
+
 
     def get_file_type(self, path):
         if path.endswith(("jpg", "jpeg", "png")):
@@ -1183,12 +1205,16 @@ class MainWindow(QtG.QMainWindow):
         self.update_tree_boxsizes(update_current=True)
 
     def _draw_all_boxes(self):
+        state = self.get_filter_state()
         for box in self.rectangles:
+            rect = box.getRect()
             if self.use_estimated_size:
-                self.resize_box(box, box.est_size)
+                self.resize_box(rect, box.est_size)
             else:
-                self.resize_box(box, self.boxsize)
-            self.ax.draw_artist(box)
+                self.resize_box(rect, self.boxsize)
+            self.ax.draw_artist(rect)
+            if not self.filter_tuple_is_equal(self.get_filter_state(),state):
+                break
         self.fig.canvas.blit(self.ax.bbox)
 
     def get_corresponding_box(self, x, y, rectangles, get_low=False):
